@@ -7,14 +7,12 @@ This defines a callable class that IPython uses for `sys.displayhook`.
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-from __future__ import print_function
-
+import builtins as builtin_mod
 import sys
 import io as _io
 import tokenize
 
 from traitlets.config.configurable import Configurable
-from IPython.utils.py3compat import builtin_mod, cast_unicode_py2
 from traitlets import Instance, Float
 from warnings import warn
 
@@ -76,6 +74,9 @@ class DisplayHook(Configurable):
         # particular uses _, so we need to stay away from it.
         if '_' in builtin_mod.__dict__:
             try:
+                user_value = self.shell.user_ns['_']
+                if user_value is not self._:
+                    return
                 del self.shell.user_ns['_']
             except KeyError:
                 pass
@@ -85,7 +86,7 @@ class DisplayHook(Configurable):
         # do not print output if input ends in ';'
         
         try:
-            cell = cast_unicode_py2(self.shell.history_manager.input_hist_parsed[-1])
+            cell = self.shell.history_manager.input_hist_parsed[-1]
         except IndexError:
             # some uses of ipshellembed may fail here
             return False
@@ -195,13 +196,23 @@ class DisplayHook(Configurable):
         if result is not self.shell.user_ns['_oh']:
             if len(self.shell.user_ns['_oh']) >= self.cache_size and self.do_full_cache:
                 self.cull_cache()
-            # Don't overwrite '_' and friends if '_' is in __builtin__ (otherwise
-            # we cause buggy behavior for things like gettext).
 
-            if '_' not in builtin_mod.__dict__:
-                self.___ = self.__
-                self.__ = self._
-                self._ = result
+            # Don't overwrite '_' and friends if '_' is in __builtin__
+            # (otherwise we cause buggy behavior for things like gettext). and
+            # do not overwrite _, __ or ___ if one of these has been assigned
+            # by the user.
+            update_unders = True
+            for unders in ['_'*i for i in range(1,4)]:
+                if not unders in self.shell.user_ns:
+                    continue
+                if getattr(self, unders) is not self.shell.user_ns.get(unders):
+                    update_unders = False
+
+            self.___ = self.__
+            self.__ = self._
+            self._ = result
+
+            if ('_' not in builtin_mod.__dict__) and (update_unders):
                 self.shell.push({'_':self._,
                                  '__':self.__,
                                 '___':self.___}, interactive=False)
@@ -209,7 +220,7 @@ class DisplayHook(Configurable):
             # hackish access to top-level  namespace to create _1,_2... dynamically
             to_main = {}
             if self.do_full_cache:
-                new_result = '_'+repr(self.prompt_count)
+                new_result = '_%s' % self.prompt_count
                 to_main[new_result] = result
                 self.shell.push(to_main, interactive=False)
                 self.shell.user_ns['_oh'][self.prompt_count] = result
