@@ -28,10 +28,7 @@ tests = [
     ('??%%hist4', ('', '??', '%%hist4', '')),
     ('?x*', ('', '?', 'x*', '')),
     ]
-if py3compat.PY3:
-    tests.append((u"Pérez Fernando", (u'', u'', u'Pérez', u'Fernando')))
-else:
-    tests.append((u"Pérez Fernando", (u'', u'', u'P', u'érez Fernando')))
+tests.append((u"Pérez Fernando", (u'', u'', u'Pérez', u'Fernando')))
 
 def test_split_user_input():
     return tt.check_pairs(split_user_input, tests)
