@@ -1,5 +1,4 @@
-"""IPython terminal interface using prompt_toolkit1"""
-from __future__ import print_function
+"""IPython terminal interface using prompt_toolkit"""
 
 import os
 import sys
@@ -8,22 +7,25 @@ from warnings import warn
 
 from IPython.core.interactiveshell import InteractiveShell, InteractiveShellABC
 from IPython.utils import io
-from IPython.utils.py3compat import PY3, cast_unicode_py2, input, string_types
+from IPython.utils.py3compat import input, cast_unicode_py2
 from IPython.utils.terminal import toggle_set_term_title, set_term_title
 from IPython.utils.process import abbrev_cwd
-from traitlets import Bool, Unicode, Dict, Integer, observe, Instance, Type, default, Enum, Union
+from traitlets import (
+    Bool, Unicode, Dict, Integer, observe, Instance, Type, default, Enum, Union,
+    Any,
+)
 
-from prompt_toolkit1.document import Document
-from prompt_toolkit1.enums import DEFAULT_BUFFER, EditingMode
-from prompt_toolkit1.filters import (HasFocus, Condition, IsDone)
-from prompt_toolkit1.history import InMemoryHistory
-from prompt_toolkit1.shortcuts import create_prompt_application, create_eventloop, create_prompt_layout, create_output
-from prompt_toolkit1.interface import CommandLineInterface
-from prompt_toolkit1.key_binding.manager import KeyBindingManager
-from prompt_toolkit1.layout.processors import ConditionalProcessor, HighlightMatchingBracketProcessor
-from prompt_toolkit1.styles import PygmentsStyle, DynamicStyle
+from prompt_toolkit.document import Document
+from prompt_toolkit.enums import DEFAULT_BUFFER, EditingMode
+from prompt_toolkit.filters import (HasFocus, Condition, IsDone)
+from prompt_toolkit.history import InMemoryHistory
+from prompt_toolkit.shortcuts import create_prompt_application, create_eventloop, create_prompt_layout, create_output
+from prompt_toolkit.interface import CommandLineInterface
+from prompt_toolkit.key_binding.manager import KeyBindingManager
+from prompt_toolkit.layout.processors import ConditionalProcessor, HighlightMatchingBracketProcessor
+from prompt_toolkit.styles import PygmentsStyle, DynamicStyle
 
-from pygments.styles import get_style_by_name
+from pygments.styles import get_style_by_name, get_all_styles
 from pygments.style import Style
 from pygments.token import Token
 
@@ -37,8 +39,6 @@ from .shortcuts import register_ipython_shortcuts
 DISPLAY_BANNER_DEPRECATED = object()
 
 
-from pygments.style import Style
-
 class _NoStyle(Style): pass
 
 
@@ -61,10 +61,7 @@ _style_overrides_linux = {
 
 def get_default_editor():
     try:
-        ed = os.environ['EDITOR']
-        if not PY3:
-            ed = ed.decode()
-        return ed
+        return os.environ['EDITOR']
     except KeyError:
         pass
     except UnicodeError:
@@ -110,9 +107,7 @@ class TerminalInteractiveShell(InteractiveShell):
             IPython own testing machinery, and emacs inferior-shell integration through elpy.
 
             This mode default to `True` if the `IPY_TEST_SIMPLE_PROMPT`
-            environment variable is set, or the current terminal is not a tty.
-
-            """
+            environment variable is set, or the current terminal is not a tty."""
             ).tag(config=True)
 
     @property
@@ -131,14 +126,12 @@ class TerminalInteractiveShell(InteractiveShell):
     ).tag(config=True)
 
     mouse_support = Bool(False,
-        help="Enable mouse support in the prompt"
+        help="Enable mouse support in the prompt\n(Note: prevents selecting text with the mouse)"
     ).tag(config=True)
 
-    # We don't load the list of styles for the help string, because loading
-    # Pygments plugins takes time and can cause unexpected errors.
     highlighting_style = Union([Unicode('legacy'), Type(klass=Style)],
         help="""The name or class of a Pygments style to use for syntax
-        highlighting. To see available styles, run `pygmentize -L styles`."""
+        highlighting: \n %s""" % ', '.join(get_all_styles())
     ).tag(config=True)
 
 
@@ -166,7 +159,7 @@ class TerminalInteractiveShell(InteractiveShell):
         help="Set the editor used by IPython (default to $EDITOR/vi/notepad)."
     ).tag(config=True)
 
-    prompts_class = Type(Prompts, help='Class used to generate Prompt token for prompt_toolkit1').tag(config=True)
+    prompts_class = Type(Prompts, help='Class used to generate Prompt token for prompt_toolkit').tag(config=True)
 
     prompts = Instance(Prompts)
 
@@ -186,10 +179,15 @@ class TerminalInteractiveShell(InteractiveShell):
         help="Automatically set the terminal title"
     ).tag(config=True)
 
+    term_title_format = Unicode("IPython: {cwd}",
+        help="Customize the terminal title format.  This is a python format string. " +
+             "Available substitutions are: {cwd}."
+    ).tag(config=True)
+
     display_completions = Enum(('column', 'multicolumn','readlinelike'),
         help= ( "Options for displaying tab completions, 'column', 'multicolumn', and "
-                "'readlinelike'. These options are for `prompt_toolkit1`, see "
-                "`prompt_toolkit1` documentation for more information."
+                "'readlinelike'. These options are for `prompt_toolkit`, see "
+                "`prompt_toolkit` documentation for more information."
                 ),
         default_value='multicolumn').tag(config=True)
 
@@ -202,12 +200,18 @@ class TerminalInteractiveShell(InteractiveShell):
              "This is in addition to the F2 binding, which is always enabled."
     ).tag(config=True)
 
+    handle_return = Any(None,
+        help="Provide an alternative handler to be called when the user presses "
+             "Return. This is an advanced option intended for debugging, which "
+             "may be changed or removed in later releases."
+    ).tag(config=True)
+
     @observe('term_title')
     def init_term_title(self, change=None):
         # Enable or disable the terminal title.
         if self.term_title:
             toggle_set_term_title(True)
-            set_term_title('IPython: ' + abbrev_cwd())
+            set_term_title(self.term_title_format.format(cwd=abbrev_cwd()))
         else:
             toggle_set_term_title(False)
 
@@ -218,7 +222,7 @@ class TerminalInteractiveShell(InteractiveShell):
         # disable `_ipython_display_`
         self.display_formatter.ipython_display_formatter.enabled = False
 
-    def init_prompt_toolkit1_cli(self):
+    def init_prompt_toolkit_cli(self):
         if self.simple_prompt:
             # Fall back to plain non-interactive output for tests.
             # This is very limited, and only accepts a single line.
@@ -323,7 +327,7 @@ class TerminalInteractiveShell(InteractiveShell):
             else :
                 raise ValueError('Got unknown colors: ', legacy)
         else :
-            if isinstance(name_or_cls, string_types):
+            if isinstance(name_or_cls, str):
                 style_cls = get_style_by_name(name_or_cls)
             else:
                 style_cls = name_or_cls
@@ -379,26 +383,7 @@ class TerminalInteractiveShell(InteractiveShell):
             return
 
         import win_unicode_console
-
-        if PY3:
-            win_unicode_console.enable()
-        else:
-            # https://github.com/ipython/ipython/issues/9768
-            from win_unicode_console.streams import (TextStreamWrapper,
-                                 stdout_text_transcoded, stderr_text_transcoded)
-
-            class LenientStrStreamWrapper(TextStreamWrapper):
-                def write(self, s):
-                    if isinstance(s, bytes):
-                        s = s.decode(self.encoding, 'replace')
-
-                    self.base.write(s)
-
-            stdout_text_str = LenientStrStreamWrapper(stdout_text_transcoded)
-            stderr_text_str = LenientStrStreamWrapper(stderr_text_transcoded)
-
-            win_unicode_console.enable(stdout=stdout_text_str,
-                                       stderr=stderr_text_str)
+        win_unicode_console.enable()
 
     def init_io(self):
         if sys.platform not in {'win32', 'cli'}:
@@ -438,7 +423,7 @@ class TerminalInteractiveShell(InteractiveShell):
 
     def __init__(self, *args, **kwargs):
         super(TerminalInteractiveShell, self).__init__(*args, **kwargs)
-        self.init_prompt_toolkit1_cli()
+        self.init_prompt_toolkit_cli()
         self.init_term_title()
         self.keep_running = True
 
@@ -454,13 +439,13 @@ class TerminalInteractiveShell(InteractiveShell):
             # We can't set the buffer here, because it will be reset just after
             # this. Adding a callable to pre_run_callables does what we need
             # after the buffer is reset.
-            s = cast_unicode_py2(self.rl_next_input)
+            s = self.rl_next_input
             def set_doc():
                 self.pt_cli.application.buffer.document = Document(s)
             if hasattr(self.pt_cli, 'pre_run_callables'):
                 self.pt_cli.pre_run_callables.append(set_doc)
             else:
-                # Older version of prompt_toolkit1; it's OK to set the document
+                # Older version of prompt_toolkit; it's OK to set the document
                 # directly here.
                 set_doc()
             self.rl_next_input = None
@@ -498,7 +483,7 @@ class TerminalInteractiveShell(InteractiveShell):
                 print("\n%s escaped interact()\n" % type(e).__name__)
             finally:
                 # An interrupt during the eventloop will mess up the
-                # internal state of the prompt_toolkit1 library.
+                # internal state of the prompt_toolkit library.
                 # Stopping the eventloop fixes this, see
                 # https://github.com/ipython/ipython/pull/9867
                 if hasattr(self, '_eventloop'):
