@@ -9,15 +9,11 @@ test suite.
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-from __future__ import print_function
 
 import argparse
-import json
 import multiprocessing.pool
 import os
 import stat
-import re
-import requests
 import shutil
 import signal
 import sys
@@ -26,32 +22,14 @@ import time
 
 from .iptest import (
     have, test_group_names as py_test_group_names, test_sections, StreamCapturer,
-    test_for,
 )
 from IPython.utils.path import compress_user
 from IPython.utils.py3compat import bytes_to_str
 from IPython.utils.sysinfo import get_sys_info
 from IPython.utils.tempdir import TemporaryDirectory
-from IPython.utils.text import strip_ansi
 
-try:
-    # Python >= 3.3
-    from subprocess import TimeoutExpired
-    def popen_wait(p, timeout):
-        return p.wait(timeout)
-except ImportError:
-    class TimeoutExpired(Exception):
-        pass
-    def popen_wait(p, timeout):
-        """backport of Popen.wait from Python 3"""
-        for i in range(int(10 * timeout)):
-            if p.poll() is not None:
-                return
-            time.sleep(0.1)
-        if p.poll() is None:
-            raise TimeoutExpired
-
-NOTEBOOK_SHUTDOWN_TIMEOUT = 10
+def popen_wait(p, timeout):
+    return p.wait(timeout)
 
 class TestController(object):
     """Run tests in a subprocess
