@@ -5,7 +5,6 @@ Authors
 - Fernando Perez <Fernando.Perez@berkeley.edu>
 """
 
-from __future__ import absolute_import
 
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
@@ -18,11 +17,7 @@ import tempfile
 from contextlib import contextmanager
 from io import StringIO
 from subprocess import Popen, PIPE
-try:
-    from unittest.mock import patch
-except ImportError:
-    # Python 2 compatibility
-    from mock import patch
+from unittest.mock import patch
 
 try:
     # These tools are used by parts of the runtime, so we make the nose
@@ -327,15 +322,7 @@ def check_pairs(func, pairs):
         assert out == expected, pair_fail_msg.format(name, inp, expected, out)
 
 
-if py3compat.PY3:
-    MyStringIO = StringIO
-else:
-    # In Python 2, stdout/stderr can have either bytes or unicode written to them,
-    # so we need a class that can handle both.
-    class MyStringIO(StringIO):
-        def write(self, s):
-            s = py3compat.cast_unicode(s, encoding=DEFAULT_ENCODING)
-            super(MyStringIO, self).write(s)
+MyStringIO = StringIO
 
 _re_type = type(re.compile(r''))
 
@@ -359,7 +346,7 @@ class AssertPrints(object):
     """
     def __init__(self, s, channel='stdout', suppress=True):
         self.s = s
-        if isinstance(self.s, (py3compat.string_types, _re_type)):
+        if isinstance(self.s, (str, _re_type)):
             self.s = [self.s]
         self.channel = channel
         self.suppress = suppress
@@ -456,9 +443,7 @@ def fake_input(inputs):
         except StopIteration:
             raise EOFError('No more inputs given')
 
-    input_name = '%s.%s' % (py3compat.builtin_mod_name,
-                            'input' if py3compat.PY3 else 'raw_input')
-    return patch(input_name, mock_input)
+    return patch('builtins.input', mock_input)
 
 def help_output_test(subcommand=''):
     """test that `ipython [subcommand] -h` works"""
