@@ -2,21 +2,10 @@
 """A fancy version of Python's builtin :func:`dir` function.
 """
 
-#-----------------------------------------------------------------------------
-#  Copyright (C) 2008-2011  The IPython Development Team
-#
-#  Distributed under the terms of the BSD License.  The full license is in
-#  the file COPYING, distributed as part of this software.
-#-----------------------------------------------------------------------------
+# Copyright (c) IPython Development Team.
+# Distributed under the terms of the Modified BSD License.
 
-#-----------------------------------------------------------------------------
-# Imports
-#-----------------------------------------------------------------------------
-from .py3compat import string_types
-
-#-----------------------------------------------------------------------------
-# Code
-#-----------------------------------------------------------------------------
+import inspect
 
 
 def safe_hasattr(obj, attr):
@@ -51,8 +40,44 @@ def dir2(obj):
         # TypeError: dir(obj) does not return a list
         words = set()
 
+    if safe_hasattr(obj, '__class__'):
+        words |= set(dir(obj.__class__))
+
     # filter out non-string attributes which may be stuffed by dir() calls
     # and poor coding in third-party modules
 
-    words = [w for w in words if isinstance(w, string_types)]
+    words = [w for w in words if isinstance(w, str)]
     return sorted(words)
+
+
+def get_real_method(obj, name):
+    """Like getattr, but with a few extra sanity checks:
+
+    - If obj is a class, ignore its methods
+    - Check if obj is a proxy that claims to have all attributes
+    - Catch attribute access failing with any exception
+    - Check that the attribute is a callable object
+
+    Returns the method or None.
+    """
+    if inspect.isclass(obj):
+        return None
+
+    try:
+        canary = getattr(obj, '_ipython_canary_method_should_not_exist_', None)
+    except Exception:
+        return None
+
+    if canary is not None:
+        # It claimed to have an attribute it should never have
+        return None
+
+    try:
+        m = getattr(obj, name, None)
+    except Exception:
+        return None
+
+    if callable(m):
+        return m
+
+    return None
