@@ -3,14 +3,12 @@
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-from __future__ import absolute_import, print_function
-
 from collections import namedtuple
 from io import StringIO
 from keyword import iskeyword
 
 from . import tokenize2
-from .py3compat import cast_unicode_py2
+
 
 Token = namedtuple('Token', ['token', 'text', 'start', 'end', 'line'])
 
@@ -31,7 +29,7 @@ def line_at_cursor(cell, cursor_pos=0):
     Parameters
     ----------
     
-    cell: text
+    cell: str
         multiline block of text
     cursor_pos: integer
         the cursor position
@@ -39,7 +37,7 @@ def line_at_cursor(cell, cursor_pos=0):
     Returns
     -------
     
-    (line, offset): (text, integer)
+    (line, offset): (string, integer)
         The line with the current cursor, and the character offset of the start of the line.
     """
     offset = 0
@@ -69,7 +67,6 @@ def token_at_cursor(cell, cursor_pos=0):
     cursor_pos : int
         The location of the cursor in the block where the token should be found
     """
-    cell = cast_unicode_py2(cell)
     names = []
     tokens = []
     call_names = []
@@ -85,7 +82,7 @@ def token_at_cursor(cell, cursor_pos=0):
         if end_line + 1 not in offsets:
             # keep track of offsets for each line
             lines = tok.line.splitlines(True)
-            for lineno, line in zip(range(start_line + 1, end_line + 2), lines):
+            for lineno, line in enumerate(lines, start_line + 1):
                 if lineno not in offsets:
                     offsets[lineno] = offsets[lineno-1] + len(line)
         
